@@ -20,50 +20,77 @@
 
 package org.emboss.jemboss.server;
 
-import org.emboss.jemboss.*;
-import org.emboss.jemboss.programs.*;
-import org.emboss.jemboss.parser.*;
+import org.emboss.jemboss.JembossParams;
+import org.emboss.jemboss.programs.RunEmbossApplication;
+import org.emboss.jemboss.programs.RunEmbossApplication2;
+import org.emboss.jemboss.parser.Ajax;
 
 import java.io.*;
 import java.util.*;
 
 /**
 *
-* Jemboss Server for SOAP
+* Jemboss Non-authenticated Server for Apache Axis (SOAP)
+* web services
 *
 */
 public class JembossServer
 {
 
-
-  private String fs = new String(System.getProperty("file.separator"));
-  private String ps = new String(System.getProperty("path.separator"));
-  private String ls = new String(System.getProperty("line.separator"));
+  /** file separator */
+  private String fs = System.getProperty("file.separator");
 
 //get paths to EMBOSS
+  /** jemboss properties */
   JembossParams jp = new JembossParams();
+  /** plplot path */
   String plplot = jp.getPlplot();
+  /** emboss data directory */
   String embossData = jp.getEmbossData();
+  /** emboss binary directory */
   String embossBin = jp.getEmbossBin();
+  /** emboss path */
   String embossPath = jp.getEmbossPath();
+  /** acd directory */
   String acdDirToParse = jp.getAcdDirToParse();
 
-  private String homeDirectory = new String(System.getProperty("user.home") + fs);
-  
-  private String username = new String(System.getProperty("user.name") + fs);
-  private String tmproot = new String("/tmp/SOAP/emboss/" + username );
+  /** user home directory */
+  private String homeDirectory = System.getProperty("user.home") + fs;
+  /** user name */
+  private String username = System.getProperty("user.name") + fs;
+  /** results directory */
+  private String tmproot  = "/tmp/SOAP/emboss/" + username;
+  /** results directory */
   private File tmprootDir = new File(tmproot);
 
-  private String[] envp = 
+  /** emboss run environment */
+  private String[] envp_emboss = 
   {
-    "PATH=" + embossPath + ps + embossBin,
+    "PATH=" + embossPath + System.getProperty("path.separator")
+            + embossBin,
     "PLPLOT_LIB=" + plplot,
-    "EMBOSS_DATA=" + embossData,
-    "HOME=" + homeDirectory
-  };
- 
-  private static Hashtable acdStore;
+// cygwin: embossdata application does not like 
+//         EMBOSS_DATA specified - it fails to 
+//         fetch files
+//  "EMBOSS_DATA=" + embossData,
+    "HOME=" + homeDirectory,
+    "EMBOSSCYGROOT=" + JembossParams.getCygwinRoot()
+//  ,"LD_LIBRARY_PATH=/usr/local/lib"
+// FIX FOR SOME SUNOS OR USE embossEnvironment
 
+  };
+
+  /** emboss run environment */
+  private String[] envp = jp.getEmbossEnvironmentArray(envp_emboss);
+ 
+
+  public JembossServer(){}
+
+  public JembossServer(String tmproot)
+  {
+    this.tmproot = tmproot;
+    tmprootDir = new File(tmproot);
+  }
 
   public String name()
   {
@@ -73,8 +100,15 @@ public class JembossServer
   public String version()
   {
     String embossCommand = new String(embossBin + "embossversion");
-    RunEmbossApplication rea = new RunEmbossApplication(embossCommand,envp,null);
-    rea.isProcessStdout();
+    RunEmbossApplication2 rea = new RunEmbossApplication2(embossCommand,
+                                                           envp,null);
+    try
+    {
+      Process p = rea.getProcess();
+      p.waitFor();
+    }
+    catch(InterruptedException iexp){}
+
     return rea.getProcessStdout();
   }
 
@@ -82,8 +116,15 @@ public class JembossServer
   public String appversion()
   {
     String embossCommand = new String(embossBin + "embossversion");
-    RunEmbossApplication rea = new RunEmbossApplication(embossCommand,envp,null);
-    rea.isProcessStdout();
+    RunEmbossApplication2 rea = new RunEmbossApplication2(embossCommand,
+                                                           envp,null);
+    try
+    {
+      Process p = rea.getProcess();
+      p.waitFor();
+    }
+    catch(InterruptedException iexp){}
+
     return rea.getProcessStdout();
   }
 
@@ -122,13 +163,13 @@ public class JembossServer
   }
 
 
-/**
-*
-* Retrieves the ACD file of an application.
-* @param  application name
-* @return Vector of containing the ACD string
-*
-*/
+  /**
+  *
+  * Retrieves the ACD file of an application.
+  * @param appName	application name
+  * @return 		ACD string
+  *
+  */
   public Vector show_acd(String appName)
   {
 
@@ -163,42 +204,58 @@ public class JembossServer
     return acd;
   }
 
-/**
-*
-* Returns the output of the EMBOSS utility wossname
-* @return wossname output
-*
-*/
+  /**
+  *
+  * Returns the output of the EMBOSS utility wossname
+  * @return 	wossname output
+  *
+  */
   public Vector getWossname()
   {
     Vector wossOut = new Vector();
     String embossCommand = new String(embossBin + "wossname -colon -auto");
-    RunEmbossApplication rea = new RunEmbossApplication(embossCommand,envp,null);
+    RunEmbossApplication2 rea = new RunEmbossApplication2(embossCommand,
+                                                           envp,null);
     wossOut.add("status");
     wossOut.add("0");
-    rea.isProcessStdout();
+    try
+    {
+      Process p = rea.getProcess();
+      p.waitFor();
+    }
+    catch(InterruptedException iexp){}
+
     wossOut.add("wossname");
     wossOut.add(rea.getProcessStdout());
 
     return wossOut;
+
   }
 
 
-/**
-*
-* Returns the help for an application as given by 'tfm'
-* @param String application name
-* @return help 
-*
-*/
+  /**
+  *
+  * Returns the help for an application as given by 'tfm'
+  * @param applName	application name
+  * @return 		help 
+  *
+  */
   public Vector show_help(String applName)
   {
-    String command = embossBin.concat("tfm " + applName + " -nomore");
-    RunEmbossApplication rea = new RunEmbossApplication(command,envp,null);
-    String helptext = "";
-    if(rea.isProcessStdout())
-      helptext = rea.getProcessStdout();
-    else
+    String command = embossBin.concat("tfm " + applName + 
+                                       " -html -nomore");
+    RunEmbossApplication2 rea = new RunEmbossApplication2(command,
+                                                     envp,null);
+
+    try
+    {
+      Process p = rea.getProcess();
+      p.waitFor();
+    }
+    catch(InterruptedException iexp){}
+
+    String helptext = rea.getProcessStdout();
+    if(helptext.equals(""))
       helptext = "No help available for this application.";
 
     Vector vans = new Vector();
@@ -209,13 +266,28 @@ public class JembossServer
   }
 
 
-/**
-*
-* Uses JNI to calculate sequence attributes using EMBOSS library call. 
-* @param sequence filename or database entry
-* @return sequence length, weight & type (protein/nucleotide)
-*
-*/
+  /**
+  *
+  * Uses JNI to calculate sequence attributes using EMBOSS library call.
+  * @param fileContent  sequence filename or database entry
+  * @param seqtype      sequence type (seqset/sequence)
+  * @param userName     username
+  * @return     	sequence length, weight & type (protein/nucleotide)
+  *
+  */
+  public Vector call_ajax(String fileContent, String seqtype, String userName)
+  {
+    return call_ajax(fileContent,seqtype);
+  }
+
+  /**
+  *
+  * Uses JNI to calculate sequence attributes using EMBOSS library call.
+  * @param fileContent  sequence filename or database entry
+  * @param seqtype      sequence type (seqset/sequence)
+  * @return     	sequence length, weight & type (protein/nucleotide)
+  *
+  */
   public Vector call_ajax(String fileContent, String seqtype)
   {
     boolean afile = false;
@@ -277,7 +349,6 @@ public class JembossServer
 
     if(ok)
     {
-      System.out.println("STATUS OK");
       vans.add("length");
       vans.add(new Integer(aj.length));
       vans.add("protein");
@@ -289,7 +360,6 @@ public class JembossServer
     }
     else
     {
-      System.out.println("STATUS NOT OK");
       vans.add("status");
       vans.add("1");
     }
@@ -301,41 +371,78 @@ public class JembossServer
   }
 
 
-/**
-*
-* Returns the databases held on the server
-* @return output from 'showdb'
-*
-*/
+  /**
+  *
+  * Returns the databases held on the server
+  * @return 	output from 'showdb'
+  *
+  */
   public Vector show_db()
   {
     Vector showdbOut = new Vector();
     String embossCommand = new String(embossBin + "showdb -auto");
-    RunEmbossApplication rea = new RunEmbossApplication(embossCommand,envp,null);
+    RunEmbossApplication2 rea = new RunEmbossApplication2(embossCommand,
+                                                          envp,null);
     showdbOut.add("status");
     showdbOut.add("0");
-    rea.isProcessStdout();
+
+    try
+    {
+      Process p = rea.getProcess();
+      p.waitFor();
+    }
+    catch(InterruptedException iexp){}
+
     showdbOut.add("showdb");
     showdbOut.add(rea.getProcessStdout());
      
+    // find available matrices
+    String dataFile[] = (new File(embossData)).list(new FilenameFilter()
+    {
+      public boolean accept(File dir, String name)
+      {
+        File fileName = new File(dir, name);
+        return !fileName.isDirectory();
+      };
+    });
+    String matrices ="";
+    for(int i=0;i<dataFile.length;i++)
+      matrices=matrices.concat(dataFile[i]+"\n");
+    showdbOut.add("matrices");
+    showdbOut.add(matrices);
+
+    // find available codon usage tables
+    
+    dataFile = (new File(embossData+fs+"CODONS")).list(new FilenameFilter()
+    {
+      public boolean accept(File dir, String name)
+      {
+        File fileName = new File(dir, name);
+        return !fileName.isDirectory();
+      };
+    });
+    matrices ="";
+    for(int i=0;i<dataFile.length;i++)
+      matrices=matrices.concat(dataFile[i]+"\n");
+    showdbOut.add("codons");
+    showdbOut.add(matrices);
+
     return showdbOut;
   }
 
 
-/**
-*
-* Private Server
-* 
-* Run an EMBOSS application
-* @param command line to run
-* @param unused
-* @param String of input files and names
-* @return output files from application run
-*
-*/
-  public Vector run_prog(String embossCommand, String options, String[] inFiles)
+  /**
+  *
+  * Run an EMBOSS application
+  * @param embossCommand	command line to run
+  * @param options		unused
+  * @param inFiles		input files and names
+  * @return 			output files from application run
+  *
+  */
+  public Vector run_prog(String embossCommand, String options, 
+                                             String[] inFiles)
   {
-    System.out.println("Running runProg now with Strings[].... " + tmproot);
     Vector result = new Vector();
 
     Hashtable inF = new Hashtable();
@@ -347,56 +454,62 @@ public class JembossServer
     return result;
   }
 
-/**
-*
-* Run an EMBOSS application
-* @param command line to run
-* @param unused
-* @param Hashtable of input files
-* @return output files from application run
-*
-*/
-  public Vector run_prog(String embossCommand, String options, Hashtable inFiles,
-                         String userName)
+  /**
+  *
+  * Run an EMBOSS application
+  * @param embossCommand        command line to run
+  * @param options              unused
+  * @param inFiles              input files and names
+  * @param userName		username
+  * @return                     output files from application run
+  *
+  */
+  public Vector run_prog(String embossCommand, String options,
+                         Vector inFiles, String userName)
   {
-    System.out.println("Running runProg now with Strings[].... " + tmproot + userName);
-
+    Hashtable hashInFiles = getHashtable(inFiles);
     tmproot = tmproot.concat(userName+fs);
-    Vector result = new Vector();
-    result = run_prog(embossCommand,options,inFiles);
-
-    return result;
+    return run_prog(embossCommand,options,hashInFiles);
+/*   return run_prog(embossCommand,options,hashInFiles,userName); */
   }
 
-
-/**
-*
-* Run an EMBOSS application
-* @param command line to run
-* @param unused 
-* @param Hashtable of input files
-* @return output files from application run
-*
-*/
-  public Vector run_prog(String embossCommand, String options, Hashtable inFiles)
+  /**
+  *
+  * Run an EMBOSS application
+  * @param embossCommand	command line to run
+  * @param options		unused 
+  * @param inFiles		input files
+  * @return 			output files from application run
+  *
+  */
+  public Vector run_prog(String embossCommand, String options, 
+                                            Hashtable inFiles)
   {
 
     Vector result = new Vector();
-//  System.out.println("Running runProg now.... " + tmproot);
-    
 
-    //disallow pathnames and multiple command constructions
-    if((embossCommand.indexOf("/") > -1) || (embossCommand.indexOf(";") > -1) ||
-       (embossCommand.indexOf("/") > -1) )
+    //disallow multiple command constructions
+    if(embossCommand.indexOf(";") > -1) 
     {
        result.add("msg");
        result.add("ERROR: Disallowed command syntax "+embossCommand);
        result.add("status");
        result.add("1");
+
        return result;
     }
 
-    Enumeration enum = inFiles.keys();
+    //trap anything that is trying to write to stdout
+    int stdIndex = embossCommand.indexOf(" stdout ");
+    if(stdIndex > -1)
+    {
+      String startCmd = embossCommand.substring(0,stdIndex+7);
+      String endCmd = embossCommand.substring(stdIndex+8);
+      embossCommand = startCmd.concat("file ");
+      embossCommand = embossCommand.concat(endCmd);
+    }
+
+    Enumeration enumFiles = inFiles.keys();
     String appl   = embossCommand.substring(0,embossCommand.indexOf(" "));
     String rest   = embossCommand.substring(embossCommand.indexOf(" "));
     embossCommand = embossBin.concat(embossCommand);
@@ -415,6 +528,7 @@ public class JembossServer
 
     Random rnd = new Random();
     String dat = new Date().toString();
+    dat = dat.replace(':','_');
 
 //get a unique project name 
     String project = new String(tmproot + appl + "_" +
@@ -426,6 +540,7 @@ public class JembossServer
 //create description file
     File desc = new File(new String(project + fs + ".desc"));
 
+    String ls = System.getProperty("line.separator");
     String descript = "";
     try
     {
@@ -438,31 +553,32 @@ public class JembossServer
       dout.println("Application: " + appl + ls + rest);
       dout.println("Started at " + dat + ls + ls + "Input files:");
   
-      while (enum.hasMoreElements())
+      while (enumFiles.hasMoreElements())
       {
-        String thiskey = (String)enum.nextElement().toString();
-        String filec = (String)inFiles.get(thiskey);
+        String thiskey = (String)enumFiles.nextElement().toString();
         File f = new File(project + fs + thiskey);
         descript = descript.concat(project+fs+thiskey);
         dout.println(project + fs + thiskey);
+
         try
         {
           f.createNewFile();
-          PrintWriter out = new PrintWriter(new FileWriter(f));
-          out.println(filec);
+          FileOutputStream out = new FileOutputStream(f);
+          out.write((byte []) inFiles.get(thiskey));
           out.close();
         }
-        catch (IOException ioe) 
+        catch(IOException ioe)
         {
-          msg = new String("Error making description file");
+          msg = new String("Error making input file");
         }
+
       }
       
       dout.close();
     }
     catch (IOException ioe) {} 
- 
-    RunEmbossApplication rea = new RunEmbossApplication(embossCommand,
+
+    RunEmbossApplication2 rea = new RunEmbossApplication2(embossCommand,
                                                envp,new File(project));
     
     result.add("cmd");
@@ -473,24 +589,32 @@ public class JembossServer
     result.add("0");
 
     if(options.toLowerCase().indexOf("interactive") > -1)
-    {
-      if(rea.isProcessStdout())
-      {
-        result.add("stdout");
-        result.add(rea.getProcessStdout());
-      }
-      createFinishedFile(project);
+    { 
+    	String stdout = "";
+        String stderr = "";
+        try
+        {
+          rea.getProcess().waitFor();
+          stdout = rea.getProcessStdout();
+          stderr = rea.getProcessStderr();
+        }
+        catch(InterruptedException iexp){}    
+        if(!stdout.equals(""))
+        {
+          result.add("stdout");
+          result.add(stdout);
+        }
+        
+        if(!stderr.equals(""))
+        {
+          result.add("stderr");
+          result.add(stderr);  
+        }
+        createFinishedFile(project);
     }
     else      //batch or background
     {
-//    try
-//    {
-//      Process p = rea.getProcess();
-//      p.waitFor();
-//      createFinishedFile(project);
-//    }
-//    catch(InterruptedException intr){}
-      JembossThread jt = new JembossThread(rea.getProcess(),project);
+      JembossThread jt = new JembossThread(rea,project);
       jt.start();
 
 //    if(jt.isAlive())
@@ -504,22 +628,20 @@ public class JembossServer
       result.add(descript+ls+"Application pending"+ls);
     }
 
-    System.out.println("JEMBOSSSERVER running");
-    
 //get the output files
-    result = loadFilesContent(projectDir,project,result);
+    result = loadFilesContent(projectDir,project,result,inFiles);
     result = loadPNGContent(projectDir,project,result);
 
     return result;
   }
 
-/**
-*
-* Creates a file named "finished" in the project directory,
-* that contains a time stamp.
-* @param String project directory name
-*
-*/ 
+  /**
+  *
+  * Creates a file named "finished" in the project directory,
+  * that contains a time stamp.
+  * @param project	project directory name
+  *
+  */ 
   private void createFinishedFile(String project)
   {
     File finished = new File(new String(project + fs + ".finished"));
@@ -533,16 +655,16 @@ public class JembossServer
   }
 
 
-/**
-*
-* Private Server
-*
-* Returns the results for a saved project.
-* @param project/directory name
-* @param unused
-* @return saved results files
-*
-*/
+  /**
+  *
+  * Returns the results for a saved project.
+  * @param project 	project directory name
+  * @param cl		unused if showing all results otherwise
+  *			this is the name of the file to display
+  * @param userName	username
+  * @return 		saved results files
+  *
+  */
   public Vector show_saved_results(String project, String cl,
                                    String userName)
   {
@@ -552,25 +674,49 @@ public class JembossServer
     return ssr;
   }
 
-/**
-*
-* Private Server
-*
-* Returns the results for a saved project.
-* @param project/directory name
-* @param unused
-* @return saved results files
-*
-*/
+  /**
+  *
+  * Returns the results for a saved project.
+  * @param project      project directory name
+  * @param cl           unused if showing all results otherwise
+  *                     this is the name of the file to display
+  * @return             saved results files
+  *
+  */
   public Vector show_saved_results(String project, String cl)
   {
     Vector ssr = new Vector();
    
     project = tmproot.concat(project);
     File projectDir = new File(project);
-    ssr = loadFilesContent(projectDir,project,ssr);
-    ssr = loadPNGContent(projectDir,project,ssr);
-        
+
+    if(cl.equals(""))
+    {
+      ssr = loadFilesContent(projectDir,project,ssr,null);
+      ssr = loadPNGContent(projectDir,project,ssr);
+    }
+    else
+    {
+      try
+      {
+        String fc = "";
+        String line;
+        File fn = new File(project + fs + cl);
+
+        if(fn.exists())
+        {
+          BufferedReader in = new BufferedReader(new FileReader(project +
+                                                       fs + cl));
+          while((line = in.readLine()) != null)
+            fc = fc.concat(line + "\n");
+        }
+  
+        ssr.add(cl);
+        ssr.add(fc);
+      }
+      catch (IOException ioe){}
+    }
+
     ssr.add("status");
     ssr.add("0");
 
@@ -581,16 +727,74 @@ public class JembossServer
   }
 
 
-/**
-*  
-* Private server
-*
-* Deletes a projects saved results.
-* @param project/directory name
-* @param unused
-* @return message
-*
-*/
+  /**
+  *
+  * Save a file to a project directory on the server.
+  * @param project      project directory to save file in
+  * @param filename     filename
+  * @param notes        text to write to file
+  * @param userName     username
+  * @return             message
+  *
+  */
+  public Vector save_project_file(String project, String filename,
+                                  String notes, String userName)
+  {
+    return save_project_file(userName+ fs +project, filename, notes);
+  }
+
+
+  /**
+  *
+  * Save a file to a project directory on the server.
+  * @param project	project directory to save file in
+  * @param filename	filename
+  * @param notes	text to write to file
+  * @param userName	username
+  * @return 		message
+  *
+  */
+  public Vector save_project_file(String project, String filename,
+                                  String notes)
+  {
+    Vector v = new Vector();
+    String msg = "OK";
+
+    String fn = tmproot + fs + 
+                     project + fs + filename;
+
+    File f = new File(fn);
+    try
+    {
+      if(!f.exists())
+        f.createNewFile();
+      FileOutputStream out = new FileOutputStream(f);
+      out.write(notes.getBytes());
+      out.close();
+    }
+    catch(IOException ioe)
+    {
+      msg = new String("Error making input file");
+    }
+
+    v.add("status");
+    v.add("0");
+    v.add("msg");
+    v.add(msg);
+
+    return v;
+  }
+
+
+  /**
+  *  
+  * Deletes a projects saved results.
+  * @param project	project directory name
+  * @param cl		unused
+  * @param userName     username
+  * @return 		message
+  *
+  */
   public Vector delete_saved_results(String project, String cl,
                                      String userName)
   {
@@ -600,28 +804,30 @@ public class JembossServer
     return dsr;
   }
 
-/**
-*
-* Private server
-*
-* Deletes a projects saved results.
-* @param project/directory name
-* @param unused
-* @return message
-*
-*/
+  /**
+  * 
+  * Deletes a projects saved results.
+  * @param project      project directory name
+  * @param cl           unused
+  * @return             message
+  *
+  */
   public Vector delete_saved_results(String project, String cl)
   {
     Vector dsr = new Vector();
 
-    project = tmproot.concat(project);
-    File projectDir = new File(project);
-    File resFiles[] = projectDir.listFiles();
-
-    for(int i=0;i<resFiles.length;i++)
-      resFiles[i].delete();
+    StringTokenizer st = new StringTokenizer(project,"\n");
+    while(st.hasMoreTokens()) 
+    {
+      String proj = tmproot.concat(fs+st.nextToken());
+    
+      File projectDir = new File(proj);
+      File resFiles[] = projectDir.listFiles();
+      for(int i=0;i<resFiles.length;i++)
+        resFiles[i].delete();
       
-    projectDir.delete();
+      projectDir.delete();
+    }
 
     dsr.add("status");
     dsr.add("0");
@@ -631,14 +837,13 @@ public class JembossServer
   }
 
 
-/**
-*
-* Private Server
-*
-* List of the saved results on the server.
-* @return list of the saved results.
-*
-*/
+  /**
+  *
+  * List of the saved results on the server.
+  * @param userName	username
+  * @return 		list of the saved results.
+  *
+  */
   public Vector list_saved_results(String userName)
   {
     Vector lsr = new Vector();
@@ -648,14 +853,12 @@ public class JembossServer
     return lsr;
   }
 
-/**
-*
-* Private Server
-*
-* List of the saved results on the server.
-* @return list of the saved results.
-*
-*/
+  /**
+  *
+  * List of the saved results on the server.
+  * @return             list of the saved results.
+  *
+  */
   public Vector list_saved_results()
   {
     Vector lsr = new Vector();
@@ -704,12 +907,19 @@ public class JembossServer
   
   }
 
-/**
-*
-* Reads in files from EMBOSS output
-*
-*/
-  private Vector loadFilesContent(File projectDir, String project, Vector result)
+
+  /**
+  *
+  * Reads in files from EMBOSS output
+  * @param projectDir   project directory
+  * @param project      project name
+  * @param result       results
+  * @param inFiles      input files
+  * @return             result
+  *
+  */
+  private Vector loadFilesContent(File projectDir, String project, 
+                                  Vector result, Hashtable inFiles)
   {
 
     String outFiles[] = projectDir.list(new FilenameFilter()
@@ -725,13 +935,34 @@ public class JembossServer
       String line = new String("");
       String fc = new String("");
       String key = new String(outFiles[i]);
-      try
+
+
+      if(inFiles != null)
       {
-        BufferedReader in = new BufferedReader(new FileReader(project + fs + outFiles[i]));
-        while((line = in.readLine()) != null)
-          fc = fc.concat(line + "\n");
+        if(!inFiles.containsKey(key))        // leave out input files
+        {
+          try
+          {
+            BufferedReader in = new BufferedReader(new FileReader(project + fs +
+                                                                  outFiles[i]));
+            while((line = in.readLine()) != null)
+              fc = fc.concat(line + "\n");
+          }
+          catch (IOException ioe){}
+        }
       }
-      catch (IOException ioe){}
+      else
+      {
+        try
+        {
+          BufferedReader in = new BufferedReader(new FileReader(project + fs +
+                                                                outFiles[i]));
+          while((line = in.readLine()) != null)
+            fc = fc.concat(line + "\n");
+        }
+        catch (IOException ioe){}
+      }
+
       if(!fc.equals(""))
       {
         result.add(key);
@@ -742,11 +973,15 @@ public class JembossServer
 
   }
 
-/**
-*
-* Reads in png files from EMBOSS output
-*
-*/
+  /**
+  *
+  * Reads in png files from EMBOSS output
+  * @param projectDir   project directory
+  * @param project      project name
+  * @param result       results
+  * @return             result
+  *
+  */
   private Vector loadPNGContent(File projectDir, String project, Vector result)
   {
 
@@ -754,7 +989,10 @@ public class JembossServer
     {
       public boolean accept(File cwd, String name)
       {
-        return name.endsWith(".png");
+        if( name.endsWith(".png") ||
+            name.endsWith(".dat") )
+          return true;
+        return false;
       };
     });
 
@@ -774,88 +1012,64 @@ public class JembossServer
     return result;
   }
 
-/**
-*
-* Read a file into a byte array.
-* @param String file name
-* @return byte[] contents of file
-*
-*/
+  /**
+  *
+  * Read a file into a byte array.
+  * @param filename	file name
+  * @return 		byte[] contents of file
+  *
+  */
   protected static byte[] readByteFile(String filename)
   {
 
-    DataInputStream dis = null;
-    FileInputStream fis = null;
-    int nby = 0;
-    byte data[] = null;
-
+    File fn = new File(filename);
+    byte[] b = null;
     try
     {
-      fis = new FileInputStream(filename);
-      dis = new DataInputStream(fis);
-      while(true)
-      {
-        dis.readByte();
-        nby++;
-      }
+      long s = fn.length();
+      if(s == 0)
+        return b;
+      b = new byte[(int)s];
+      FileInputStream fi = new FileInputStream(fn);
+      fi.read(b);
+      fi.close();
     }
-    catch (EOFException eof){}
-    catch (IOException ioe){}
-    finally
+    catch (IOException ioe)
     {
-      if(dis!=null)
-        try
-        {
-          dis.close();
-        }
-        catch (IOException ioe2){}
-      if(fis!=null)
-        try
-        {
-          fis.close();
-        }
-        catch (IOException ioe2){}
+      System.out.println("Cannot read file: " + filename);
     }
+    return b;
 
-    if(nby >0)
-    {
-      try
-      {
-        data = new byte[nby];
-        fis = new FileInputStream(filename);
-        dis = new DataInputStream(fis);
-        nby=0;
-        dis.readFully(data);
-      }
-      catch (EOFException eof){}
-      catch (IOException ioe){}
-      finally
-      {
-        if(dis!=null)
-          try
-          {
-            dis.close();
-          }
-          catch (IOException ioe2){}
-        if(fis!=null)
-          try
-          {
-            fis.close();
-          }
-          catch (IOException ioe2){}
-      }
-    }
-
-    return data;
   }
 
 
-/**
-*
-* Used to provide information on the batch/background
-* processes.
-*
-*/
+  /**
+  *
+  * Used to provide information on the batch/background
+  * processes.
+  * @param prog         program
+  * @param opt          options
+  * @param resToQuery   results to query
+  * @param userName     username
+  *
+  */
+  public Vector update_result_status(String prog, String opt,
+                        Vector resToQuery,String userName)
+  {
+    return update_result_status(prog,opt,getHashtable(resToQuery),
+                                userName);
+  }
+
+  /**
+  *
+  * Used to provide information on the batch/background
+  * processes.
+  * @param prog         program
+  * @param opt          options
+  * @param resToQuery   results to query
+  * @param userName     username
+  *
+  */
   public Vector update_result_status(String prog, String opt,
                         Hashtable resToQuery,String userName)
   {
@@ -863,21 +1077,24 @@ public class JembossServer
     return update_result_status(prog, opt, resToQuery);
   }
 
-/**
-*
-* Used to provide information on the batch/background 
-* processes.
-*
-*/
+  /**
+  *
+  * Used to provide information on the batch/background
+  * processes.
+  * @param prog         program
+  * @param opt          options
+  * @param resToQuery   results to query
+  *
+  */
   public Vector update_result_status(String prog, String opt, 
                                      Hashtable resToQuery)
   {
 
     Vector vans = new Vector();
-    Enumeration enum = resToQuery.keys();
-    while (enum.hasMoreElements())
+    Enumeration enumRes = resToQuery.keys();
+    while (enumRes.hasMoreElements())
     {
-      String thiskey = (String)enum.nextElement().toString();
+      String thiskey = (String)enumRes.nextElement().toString();
       String thiselm = (String)resToQuery.get(thiskey);
 //    System.out.println("KEY : "+thiskey+" ELEMENT: "+thiselm);
       File f = new File(tmproot+fs+thiskey+fs+".finished");
@@ -911,6 +1128,28 @@ public class JembossServer
     return vans;
   }
 
+  /**
+  *
+  * Convert contents from a Vector to a Hashtable
+  * @param v    Vector
+  *
+  */
+  private Hashtable getHashtable(Vector v)
+  {
+    Hashtable h = new Hashtable();
+    for(Enumeration e = v.elements() ; e.hasMoreElements() ;)
+    {
+      String s = (String)e.nextElement();
+      h.put(s,e.nextElement());
+    }
+    return h;
+  }
+
+
+  public final Object clone() throws java.lang.CloneNotSupportedException 
+  {
+    throw new java.lang.CloneNotSupportedException();
+  }
 
 }
 
